/*************************************************************************
Copyright (c) 2010-2011, Valentina BOEVA.

>>> SOURCE LICENSE >>>
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation (www.fsf.org); either version 2 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

A copy of the GNU General Public License is available at
http://www.fsf.org/licensing/licenses

>>> END OF LICENSE >>>
*************************************************************************/

#include "SNPinGenome.h"

using namespace std ;

SNPinGenome::SNPinGenome(std::string const& inFile)
{
    cout << "..Starting reading "<< inFile << " to get SNP positions\n";
	std::ifstream fileSNP (inFile.c_str());

    if (!fileSNP.is_open()) {
	    cerr << "Error: unable to open "+inFile+"\n" ;
	    exit(-1);
	}
    int count = 0;
   	string line;

    int index = 0;
    string myChr = "1";
    SNP_atChr_.push_back(SNPatChr("1"));

    int previousPos = NA;

	while (std::getline(fileSNP,line)) {

        if (! line.length()) continue;

//chr1	11273	C/G	rs72481019

		std::vector<std::string> strs = split(line, '\t');
		if (strs.size()>=3) {

			string chr = strs[0];
            processChrName(chr);
            if (chr.compare(myChr)!=0) {
                index++;
                SNP_atChr_.push_back(SNPatChr(chr));
                myChr = chr;

            }
            int position = atoi(strs[1].c_str());
            if (position!=previousPos) {
                SNP_atChr_[index].push_SNP(SNPposition(position,strs[2],strs[3],strs[4]));
                count++;
                previousPos = position;
            }
        }

		strs.clear();
    }
    cout << "..read " << count <<" SNP positions\n";
	fileSNP.close();



}

SNPinGenome::~SNPinGenome()
{
    //dtor
}

void SNPinGenome::processPileUPLine(int & positionCount, string line, string & oldChr, int & sNPpositionToProceed,int minimalTotalLetterCountPerPosition,int & index) {

    if (! line.length()) return;;
    if (line[0] == '#') return;;
    if ( line[0] == '@') return;;

    //we will count letters after . and , in corresponding positions:
    std::vector<std::string> strs = split(line, '\t');
    if (strs.size() <=4) {
        strs.clear();
        return;
    }
    string chr = strs[0];
    if (chr.compare(oldChr)!=0) {
            oldChr = chr;
            processChrName(chr);
            index = findIndex(chr);
            positionCount = 0;
            if ( index == NA) {
                strs.clear();
                cout << "will skip chr"<<chr<<"\n";
                return;
            }
            sNPpositionToProceed = SNP_atChr_[index].getPositionAt(positionCount);
    }

    if ( index == NA) {
            strs.clear();
            return;
    }

    if (atoi(strs[1].c_str())==sNPpositionToProceed) {
        try {
            float localBAF=addInfoFromAPileUp(atoi(strs[3].c_str()),minimalTotalLetterCountPerPosition,SNP_atChr_[index].getNucleotideAt(positionCount),
                                           index,positionCount,sNPpositionToProceed,strs[4].c_str());
//                    if (localBAF>=0) {
//                        heterozygousBAFs.push_back(localBAF);
//                        heterozygousBAFs05.push_back(fabs(localBAF-0.5));
//                        heterozygousBAFposition.push_back(sNPpositionToProceed);
//                    }
        } catch (const char * error){
            //will simply forget about this SNP position..
        }
    }
    if (atoi(strs[1].c_str())>sNPpositionToProceed && sNPpositionToProceed != NA) {
            while (atoi(strs[1].c_str())>sNPpositionToProceed && sNPpositionToProceed != NA) {
                SNP_atChr_[index].setValueAt(positionCount,NA);
                positionCount++;
                if (positionCount<SNP_atChr_[index].getSize())
                    sNPpositionToProceed = SNP_atChr_[index].getPositionAt(positionCount);
                else
                    sNPpositionToProceed=NA;
            }
            if (atoi(strs[1].c_str())==sNPpositionToProceed) {
                 try {
                    float localBAF=addInfoFromAPileUp(atoi(strs[3].c_str()),minimalTotalLetterCountPerPosition,SNP_atChr_[index].getNucleotideAt(positionCount),
                                           index,positionCount,sNPpositionToProceed,strs[4].c_str());
//                        if (localBAF>=0) {
//                            heterozygousBAFs.push_back(localBAF);
//                            heterozygousBAFs05.push_back(fabs(localBAF-0.5));
//                            heterozygousBAFposition.push_back(sNPpositionToProceed);
//
//                        }
                } catch (const char * error){
                    //will simply forget about this SNP position..
                }
            }
    }
    strs.clear();
}

void SNPinGenome::assignValues(std::string const& inFile, string inputFormat, int minimalTotalLetterCountPerPosition) {
//    vector <float> heterozygousBAFs;
//    vector <float> heterozygousBAFs05;
//    vector <int> heterozygousBAFposition;

    string oldChr = "azeaze";

    int sNPpositionToProceed;
    int positionCount = 0;
    string line;
    int index=NA;

    if ((inputFormat.compare("pileup")==0 || inputFormat.compare("SAMtools pileup")==0)){
        //cout << "..will not proceed with BAF profiles: wrong inputFormat \n";
        cout << "..use \"pileup\" format of reads to calculate BAF profiles\n";

        cout << "..Starting reading "<< inFile << " to calculate BAF profiles\n";

        if (inFile.substr(inFile.size()-3,3).compare(".gz")==0) {
            FILE *stream;
            char buffer[MAX_BUFFER];
            string command = "gzip -cd "+inFile;
            stream =
            #ifdef _WIN32
				_popen(command.c_str(), "r");
			#else
				popen(command.c_str(), "r");
			#endif
            while ( fgets(buffer, MAX_BUFFER, stream) != NULL ) {
                line = buffer;
                processPileUPLine(positionCount, line, oldChr, sNPpositionToProceed,minimalTotalLetterCountPerPosition,index);

            }
            #ifdef _WIN32
                    _pclose(stream);
            #else
                    pclose(stream);
            #endif

        } else {
            std::ifstream fileMates (inFile.c_str());
            while (std::getline(fileMates,line)) {
                processPileUPLine(positionCount, line, oldChr, sNPpositionToProceed,minimalTotalLetterCountPerPosition,index);
            }
            fileMates.close();
        }



        //BEGIN TMP:
//        std::vector<int> bpfinal;
//        int tmp = calculateBreakpoints_general(-0.0001, heterozygousBAFs05.size(), heterozygousBAFs05, bpfinal, 0, 4);
//        bpfinal.push_back(heterozygousBAFs05.size());
//        std::vector<float> medians;
//        std::vector<float> medianBAFs;
//
//        int start = 0;
//        for (int i = 0; i<bpfinal.size(); i++) {
//            float localMedian = get_median(heterozygousBAFs05, start, bpfinal[i]);
//            medians.push_back(localMedian);
//            for (int j = start; j<bpfinal[i];j++) {
//                medianBAFs.push_back(localMedian+0.5);
//            }
//            start = bpfinal[i];
//        }
//        std::ofstream file ("/bioinfo/users/vboeva/Desktop/Neuroblastome/ANALYSES_CNG/Valentina_analysis/freec/rowBAFS.txt");
//        for (int i = 0; i< heterozygousBAFs.size(); i++)
//            file << heterozygousBAFs[i]<<"\t"<<medianBAFs[i] <<"\t"<<heterozygousBAFs05[i]<<"\t"<<heterozygousBAFposition[i]<<"\n";
//        file.close();
//        heterozygousBAFs.clear();
//        medianBAFs.clear();
//        heterozygousBAFs05.clear();
//        heterozygousBAFposition.clear();
         //END TMP:
    } else {//if (inputFormat.compare("eland")==0 || inputFormat.compare("Eland")==0) {
        cerr << "\nWrong data formal: "<< inputFormat << "\n Currently, Control-FREEC only accepts SAM pileup files to calculate BAF profiles\n";
        exit(1);
    }


}

int SNPinGenome::findIndex (string chromosome) {
    for (int unsigned i = 0; i < SNP_atChr_.size(); i++) {
        if (chromosome.compare(SNP_atChr_[i].getChromosome())==0) {
            return i;
        }
    }
    return NA;
}

SNPatChr SNPinGenome::SNP_atChr(int index) {
    return SNP_atChr_[index];
}

float SNPinGenome::addInfoFromAPileUp (int totalLetterCount, int minimalTotalLetterCountPerPosition,char whatToLook, int index,int &positionCount, int &sNPpositionToProceed, const char * pileup){
    float value=NA;
    float status = NA;

    if (totalLetterCount>=minimalTotalLetterCountPerPosition) {
        //check that there is not deletions/insertions at this position:
        if (strccnt(pileup, int('+'))+strccnt(pileup, int('-'))==0) {
            //we will recalculate totalLetterCount = refCount+countForOverLetter
            int refCount =strccnt(pileup, '.')+strccnt(pileup, ',');
            int countForOverLetter=strccnt(pileup, whatToLook)+strccnt(pileup, tolower(whatToLook));

            //value = countForOverLetter*1./totalLetterCount;

            if (refCount+countForOverLetter>=minimalTotalLetterCountPerPosition) {
                if (countForOverLetter+refCount>totalLetterCount) {
                        countForOverLetter=totalLetterCount-refCount;
                }
                value = countForOverLetter*1./(refCount+countForOverLetter);
                //value = fabs(countForOverLetter*1./totalLetterCount-0.5);
              //  cout << "pileup: "<< pileup << " ; whatToLook: "<< whatToLook<< "\n";
               // cout << "countForOverLetter-1: "<< countForOverLetter-1 << "; totalLetterCount: "<< totalLetterCount << "; ERROR_PER_POS: "<< ERROR_PER_POS << "\n";
                double prob = binomialcdistribution(countForOverLetter-1, totalLetterCount, ERROR_PER_POS); //probability that there > k Beta Allel bases if the SNP is HomoZ.
                status = 0;
                if (prob<0.05) { //0.01

                    if (binomialcdistribution(totalLetterCount-countForOverLetter-1, totalLetterCount, ERROR_PER_POS)<0.01)
                        //decide that SNP is not homoZ.
                        status = 0.5; //at this point, to indicate possible AB
                }
            }

        }
    }

    SNP_atChr_[index].setValueAt(positionCount,value);
    SNP_atChr_[index].setStatusAt(positionCount,status);

    positionCount++;
    if (positionCount<SNP_atChr_[index].getSize())
        sNPpositionToProceed = SNP_atChr_[index].getPositionAt(positionCount);
    else
        sNPpositionToProceed=NA;
    if (status>0)
        return value;
    return NA;
}
